package org.bitbucket.bradleysmithllc.etlunit.metadata.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.lang3.ObjectUtils;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataArtifactContent;
import org.bitbucket.bradleysmithllc.etlunit.metadata.TestReference;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;

import javax.inject.Inject;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class MetaDataArtifactContentImpl implements MetaDataArtifactContent
{
	private RuntimeSupport runtimeSupport;
	private String name;
	private final List<TestReference> references = new ArrayList<TestReference>();

	public MetaDataArtifactContentImpl() {
	}

	public MetaDataArtifactContentImpl(String name) {
		this.name = name;
	}

	@Inject
	public void receiveRuntimeSupport(RuntimeSupport support)
	{
		runtimeSupport = support;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public List<TestReference> getReferences() {
		return Collections.unmodifiableList(references);
	}

	@Override
	public TestReference referencedByCurrentTest(String classification) {
		if (runtimeSupport == null)
		{
			throw new IllegalStateException("Runtime support not available");
		}

		return referencedBy(classification, runtimeSupport.getCurrentlyProcessingTestMethod());
	}

	@Override
	public TestReference referencedBy(String classification, ETLTestMethod method) {
		TestReference ref = new TestReferenceImpl(
			classification,
			method.getTestClass().getPackage(),
			method.getTestClass().getName(),
			method.getName(),
			method.getQualifiedName()
		);

		references.add(ref);

		return ref;
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException {
		writer.name("name").value(name);

		writer.name("referencedBy");
		writer.beginArray();

		for (TestReference ref : references)
		{
			writer.beginObject();
			ref.toJson(writer);
			writer.endObject();
		}

		writer.endArray();
	}

	@Override
	public void fromJson(JsonNode node) {
		references.clear();

		name = node.get("name").asText();

		ArrayNode refsNode = (ArrayNode) node.get("referencedBy");

		Iterator<JsonNode> refs = refsNode.elements();

		while (refs.hasNext())
		{
			TestReferenceImpl timpl = new TestReferenceImpl();

			timpl.fromJson(refs.next());

			references.add(timpl);
		}
	}

	public boolean equals(Object other)
	{
		if (getClass() != other.getClass())
		{
			return false;
		}

		MetaDataArtifactContentImpl oImpl = (MetaDataArtifactContentImpl) other;

		return ObjectUtils.compare(name, oImpl.name) == 0
				&& references.equals(oImpl.references);
	}
}